local logFile = io.open("/var/log/hotspot.log", "a")
function log(data)
  if (_G["DEBUG"] == nil) then
    return
  end
  logFile:write("\n" .. os.date("%Y/%m/%d %H:%M:%S "))
  if type(data) == "table" then
    logFile:write("\n")
    logFile:write(table2String(data, "  "))
  else
    logFile:write(tostring(data))
  end
end

function cmdCall(cmd)
  log("call cmd")
  log(cmd)

  local handler = io.popen(cmd)
  local output = handler:read("*a")
  handler:close()
  return output
end

function requestCall(data)
  log("call request")
  log(data)

  local cmd = "wget -T 5 -qO - "

  if type(data) == "string" then
    cmd = cmd .. "'" .. data .. "'"
  else
    if type(data.headers) == "table" then
      for k, v in pairs(data.headers) do
        log({k = k, v = v})
        cmd = cmd .. "--header='" .. k .. ": " .. v .. "' "
      end
    end

    cmd = cmd .. "'" .. data.url .. "'"
  end

  local success, out = pcall(cmdCall, cmd)
  log(out)

  return 200, out
end

local hotspotBasePort = 8079
local autoLoginBasePort = 7999

function getToken()
  local success, out = pcall(cmdCall, "otp")
  return out
end

function getAuthHeader()
  local headers = {deviceId = _G["DEVICE_ID"]}
  headers["device-token"] = getToken()
  return headers
end

function getSsid(env)
  local port = tonumber(env.SERVER_PORT)
  local basePort = autoLoginBasePort
  if (port > hotspotBasePort) then
    basePort = hotspotBasePort
  end

  return port - basePort
end

function getMac(env, retry)
  if env.mac ~= nil then
    return env.mac
  end

  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)

  if retry == nil then
    retry = 1
  end
  if retry > 3 then
    log({ip = ip, message = "Cannot get mac after 3 times"})
    return nil
  end

  local inf = "br-wifi" .. ssid
  local handle =
    io.popen(
    "ip neigh show dev " ..
      inf ..
        " | grep -v FAILED | grep '" .. ip .. " ' | awk '{printf toupper($3)}'"
  )
  local mac = handle:read("*a")
  handle:close()

  if mac ~= nil then
    env.mac = mac
    return mac
  else
    return getMac(env, retry + 1)
  end
end

function getServerHost(env)
  local ssid = getSsid(env)
  local ip = _G["SSID" .. ssid .. "_IP"]

  if
    _G["SSID" .. ssid .. "_HOSTNAME"] and
      _G["SSID" .. ssid .. "_HOSTNAME"] ~= ""
   then
    ip = _G["SSID" .. ssid .. "_HOSTNAME"]
  end

  return ip
end

function isServerRequest(env, url)
  local ssid = getSsid(env)
  local ip = _G["SSID" .. ssid .. "_IP"]
  local host = _G["SSID" .. ssid .. "_HOSTNAME"] or ""

  return string.find(url, ip) or string.find(url, host)
end

function getRedirectUrl(env)
  dofile("/www/config/template.conf")
  local ssid = getSsid(env)

  local redirect_url
  local query = parseQueryString(env.QUERY_STRING or "")

  if query.url then
    redirect_url = decodeURIComponent(query.url)
  elseif query.redirect then
    redirect_url = decodeURIComponent(query.redirect)
  else
    redirect_url = _G["REDIRECT_SSID" .. ssid]
  end

  return (redirect_url or "https://meganet.com.vn")
end

function isLoggedIn(env)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  local code = os.execute("ipset test loginlist" .. ssid .. " " .. ip)
  return code == 0
end

function isIOSCnaRequest(env)
  local ua = env.HTTP_USER_AGENT
  local isIOS =
    string.find(ua, "CaptiveNetworkSupport") or
    ((string.find(ua, "iPhone") or string.find(ua, "iPad") or
      string.find(ua, "iPod")) and
      not string.find(ua, "Safari"))
  local isIOS10dot4 = string.find(ua, "10.4.2")

  return isIOS and not isIOS10dot4
end

function isAndroidCnaRequest(env)
  local ua = env.HTTP_USER_AGENT
  return string.find(ua, "Android") and
    (string.find(ua, " wv") or env.headers["x-requested-with"] or
      env.headers["X-Requested-With"] or
      string.find(ua, "Version/%d.%d"))
end

function isCnaRequest(env)
  return isIOSCnaRequest(env) --or isAndroidCnaRequest(env)
end

function isCnaRequestBypassed(env)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)

  local handle =
    io.popen("ipset test bypass" .. ssid .. " " .. ip .. " 2>&1 | grep NOT")
  return handle:read("*a") == ""
end

function isAuthenticationRequired(env)
  local ssid = getSsid(env)
  return _G["SSID" .. ssid .. "_AUTHENTICATE_REQUIRE"] == "1"
end

function isAutoLoginEnabled(env)
  local ssid = getSsid(env)
  return _G["SSID" .. ssid .. "_AUTO_LOGIN"] == "1"
end

local sessionFields = {"id", "startedAt"}
local limitationFields = {
  "expireDate",
  "sessionTimeout",
  "dataRemaining",
  "timeRemaining",
  "maxDownload",
  "maxUpload"
}

local function parseSession(res)
  local data = {}

  if (type(res.session) == "table") then
    local session = res.session
    for i, v in ipairs(sessionFields) do
      if (session[v] ~= nil) then
        data[v] = session[v]
      end
    end
  end

  if (type(res.limitations) == "table") then
    local limitations = res.limitations
    for i, v in ipairs(limitationFields) do
      if (limitations[v] ~= nil) then
        data[v] = limitations[v]
      end
    end
  end

  return data
end

function _bypass(env, res)
  local ip = env.REMOTE_ADDR
  local ssid = getSsid(env)
  local mac = getMac(env)

  local maxDownload = ""
  if (_G["SSID" .. ssid .. "_QOS"] == "1") then
    maxDownload = _G["SSID" .. ssid .. "_QOS_DOWN"] or ""
  end

  local loginData = {
    ip = ip,
    mac = mac,
    ssid = ssid,
    idleTimeout = (_G["SSID" .. ssid .. "_USER_IDLE_TIMEOUT"] or 900),
    maxDownload = (res.maxDownload or maxDownload)
  }
  local sessionData = parseSession(res)
  log(loginData)
  log(sessionData)

  local cmd =
    table2Qs(loginData, " ", "=", true) ..
    " " ..
      table2Qs(sessionData, " ", "=", true) .. " /www/script/session/bypass.sh"

  pcall(cmdCall, cmd)
end

function callGetCurrentSession(env)
  local mac = getMac(env)
  if not mac or mac == "" then
    return {}
  end

  local queryData = table2Qs({mac = mac})
  local url = _G["AUTH_ADDRESS"] .. "/current?" .. queryData
  local requestData = {
    url = url,
    headers = getAuthHeader()
  }

  local status, code, res = pcall(requestCall, requestData)
  log(code)
  if status == false or res == "" or not res then
    -- request send failed
    res = {}
    log("call get current session failed")
  else
    res = JSON:decode(res)
    log(res)
    log("2")
  end

  return res
end

function handleLoggedIn(env, responseCna)
  if responseCna then
    if isIOSCnaRequest(env) then
      log("response ios cna")

      print("Content-Type: text/html")
      print("Connection: Close")
      print("")
      print(
        "<HTML><HEAD><TITLE>Success</TITLE></HEAD><BODY>Success</BODY></HTML>"
      )
      return
    end
    if isAndroidCnaRequest(env) then
      log("response android cna")

      httpStatus("204 No Content")
      print("Connection: Close")
      print("")
      return
    end
  end

  local url = (env.headers["referer"] or env.headers["Referer"])
  if not url or isServerRequest(env, url) then
    url = getRedirectUrl(env)
  end

  log("redirect to " .. url)
  return redirect(url)
end

function getMacAp()
  local handle = io.popen("cat /sys/class/net/eth0/address")
  return handle:read("*a")
end
